#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
@author: saaj
'''


import os.path

import pagecalc
import cherrypy
import jinja2


config = {
  'global' : {
    'server.socket_host' : '127.0.0.1',
    'server.socket_port' : 8008,
    'server.thread_pool' : 4
  },
  'pagination' : {
    'items' : 10, # items per page
    'pages' : 5,  # pages to display
  },
  '/' : {
    'tools.encode.encoding' : 'utf-8'
  }
}

path = os.path.abspath(os.path.dirname(__file__))
view = jinja2.Environment(loader = jinja2.FileSystemLoader(path))
view.globals['url'] = cherrypy.url
template = view.get_template('index.html')


class Model:
  
  _data = [i**2 for i in range(100)]
  
  
  def count(self):
    return len(self._data)
  
  def list(self, page, limit):
    return self._data[(page - 1) * limit:page * limit]
  

class Controller:
  
  @cherrypy.expose
  def index(self):
    return template.render()
  
  def example(self, page, style):
    model = Model()
    
    pagination = config['pagination']
    paginator  = pagecalc.Paginator(model.count(), pagination['items'], pagination['pages'], style)
    
    return template.render(
      data      = model.list(page, pagination['items']),
      paginator = paginator.paginate(page)
    )
  
  @cherrypy.expose
  def jumping(self, page = 1):
    return self.example(int(page), pagecalc.jumping)
  
  @cherrypy.expose
  def sliding(self, page = 1):
    return self.example(int(page), pagecalc.sliding)


if __name__ == '__main__':
  cherrypy.quickstart(Controller(), config = config)



from distutils.core import setup

setup(
  name             = 'PageCalc',
  version          = '0.2.0',
  author           = 'saaj',
  author_email     = 'mail@saaj.me',
  packages         = ['pagecalc'],
  package_data     = {'pagecalc' : ['example/*']},  
  url              = 'http://code.google.com/p/pagecalc/',
  license          = 'LGPL',
  description      = 'Python pagination calculator',
  long_description = open('README.txt').read(),
  platforms        = ['Any'],
  classifiers      = [
    'Topic :: Utilities',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Intended Audience :: Developers'
  ]
)


# -*- coding: utf-8 -*-
'''
@author: saaj
'''


from math import ceil


def sliding(paginator, page):
  '''Sliding window pagination style. Example:
        [1] 2  3  4  » →
    ← «  1 [2] 3  4  » →
    ← «  2 [3] 4  5  » →
    ← «  3 [4] 5  6  » →
    ← «  4 [5] 6  7  » →
    ← «  4  5 [6] 7  » →
    ← «  4  5  6 [7]
  '''
  
  length = min(paginator.length, paginator.pages)
  center = int(ceil(length / 2.0))
  
  # enough space to put window with centered active page 
  if page - center > paginator.pages - length:
    start = paginator.pages - length + 1
  else:
    start = max(page - center, 0) + 1

  return range(start, min(start + length, paginator.pages + 1))

def jumping(paginator, page):
  '''Jumping window pagination style. Example:
        [1] 2  3  4  » →
    ← «  1 [2] 3  4  » →
    ← «  1  2 [3] 4  » →
    ← «  1  2  3 [4] » →
    ← « [5] 6  7     » →
    ← «  5 [6] 7     » →
    ← «  5  6 [7]    
  '''
  
  steps = page % paginator.length or paginator.length
  start = page - steps + 1
  return range(start, min(start + paginator.length, paginator.pages + 1))


class Paginator(object):
  
  length = None
  '''Displayed pages number -- window length, default is 10'''
  
  style = None
  '''Pagination style, module provides ``sliding`` and ``jumping``, default is ``sliding``'''
  
  total = None
  '''Total number of items'''
   
  by = None
  '''Items per page to paginate by'''
  
  
  def __init__(self, total, by, length = 10, style = sliding):
    self.total  = total
    self.by     = by
    self.length = length
    self.style  = style
      
  @property
  def pages(self):
    '''Computes total page number'''
    
    assert self.total >= 0
    assert self.by > 0
    
    return int(ceil(self.total / float(self.by)))
    
  def paginate(self, page = 1):
    assert self.length > 0
    
    assert page > 0
    page = min(page, self.pages)
    
    pageResult = {
      'previous': page - 1 if page > 1 else None,
      'first'   : 1,
      'count'   : self.pages,
      'current' : page,
      'range'   : self.style(self, page),
      'last'    : self.pages if self.pages else 1,
      'next'    : page + 1 if page < self.pages else None,
    } 
    itemResult = {
      'totalCount' : self.total,
      'pageCount'  : self.by if page * self.by < self.total else self.total % self.by,
      'perPage'    : self.by
    }
    
    return dict(page = pageResult, item = itemResult)




'''
@author: saaj
'''


import unittest

import pagecalc


class TestPaginator(unittest.TestCase):
  
  def setUp(self):
    self.testee = pagecalc.Paginator(total = 987, by = 20) 
  
  def testPages(self):
    self.assertEqual(50, self.testee.pages)
    
    self.testee.by = 3 
    self.assertEqual(329, self.testee.pages)
    
    self.testee.total = 988
    self.assertEqual(330, self.testee.pages)
    
    self.testee.by = 1 
    self.assertEqual(988, self.testee.pages)
    
    self.testee.total = 1
    self.assertEqual(1, self.testee.pages)
    
    self.testee.total = 0
    self.assertEqual(0, self.testee.pages)
    
    self.testee.by = 0
    self.assertRaises(AssertionError, lambda: self.testee.pages)
    
    self.testee.by    = 1
    self.testee.total = -1
    self.assertRaises(AssertionError, lambda: self.testee.pages)
    
  def testSliding(self):
    self.testee.style  = pagecalc.sliding
    self.testee.length = 8
    
    expected = {
      'item' : {
        'pageCount'  : 20, 
        'perPage'    : 20,
        'totalCount' : 987
      },
      'page' : {
        'count'    : 50,
        'current'  : 44,
        'first'    : 1,
        'last'     : 50,
        'next'     : 45,
        'previous' : 43,
        'range'    : [41, 42, 43, 44, 45, 46, 47, 48]
      }
    } 
    self.assertEqual(expected, _listPageRange(self.testee.paginate(44)))
    
    expected['page'] = {
      'count'    : 50,
      'current'  : 1,
      'first'    : 1,
      'last'     : 50,
      'next'     : 2,
      'previous' : None,
      'range'    : [1, 2, 3, 4, 5, 6, 7, 8]              
    }
    self.assertEqual(expected, _listPageRange(self.testee.paginate(1)))
    
    expected['page'] = {
      'count'    : 50,
      'current'  : 3,
      'first'    : 1,
      'last'     : 50,
      'next'     : 4,
      'previous' : 2,
      'range'    : [1, 2, 3, 4, 5, 6, 7, 8]              
    }
    self.assertEqual(expected, _listPageRange(self.testee.paginate(3)))
    
    expected['page'] = {
      'count'    : 50,
      'current'  : 5,
      'first'    : 1,
      'last'     : 50,
      'next'     : 6,
      'previous' : 4,
      'range'    : [2, 3, 4, 5, 6, 7, 8, 9]              
    }
    self.assertEqual(expected, _listPageRange(self.testee.paginate(5)))
    
    expected['item']['pageCount'] = 7
    expected['page'] = {
      'count'    : 50,
      'current'  : 50,
      'first'    : 1,
      'last'     : 50,
      'next'     : None,
      'previous' : 49,
      'range'    : [43, 44, 45, 46, 47, 48, 49, 50]              
    }
    self.assertEqual(expected, _listPageRange(self.testee.paginate(50)))    
    
    self.assertEqual(expected, _listPageRange(self.testee.paginate(65)))    
    
  def testJumping(self):
    self.testee.style  = pagecalc.jumping
    self.testee.length = 8
    
    expected = {
      'item' : {
        'pageCount'  : 20, 
        'perPage'    : 20,
        'totalCount' : 987
      },
      'page' : {
        'count'    : 50,
        'current'  : 44,
        'first'    : 1,
        'last'     : 50,
        'next'     : 45,
        'previous' : 43,
        'range'    : [41, 42, 43, 44, 45, 46, 47, 48]
      }
    } 
    self.assertEqual(expected, _listPageRange(self.testee.paginate(44)))
    
    for p in range(1, 9):
      expected['page'] = {
        'count'    : 50,
        'current'  : p,
        'first'    : 1,
        'last'     : 50,
        'next'     : p + 1,
        'previous' : p - 1 if p > 1 else None,
        'range'    : [1, 2, 3, 4, 5, 6, 7, 8]              
      }
      self.assertEqual(expected, _listPageRange(self.testee.paginate(p)))
      
    expected['page'] = {
      'count'    : 50,
      'current'  : 9,
      'first'    : 1,
      'last'     : 50,
      'next'     : 10,
      'previous' : 8,
      'range'    : [9, 10, 11, 12, 13, 14, 15, 16]              
    }
    self.assertEqual(expected, _listPageRange(self.testee.paginate(9)))
    
    expected['item']['pageCount'] = 7
    expected['page'] = {
      'count'    : 50,
      'current'  : 50,
      'first'    : 1,
      'last'     : 50,
      'next'     : None,
      'previous' : 49,
      'range'    : [49, 50]              
    }
    self.assertEqual(expected, _listPageRange(self.testee.paginate(50)))
    
    self.assertEqual(expected, _listPageRange(self.testee.paginate(65)))
    
    
def _listPageRange(result):
  '''As py3 returns range class from ``range()`` it needs to be converted'''
  
  result['page']['range'] = list(result['page']['range'])
  return result
  
  